# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .Ballot import Ballot
from .GameResult import GameResult
from .options.WrittenBallotOptions import WrittenBallotOptions


class WrittenBallot:
    ballot: Ballot
    game_results: Optional[List[GameResult]] = None

    def __init__(
        self,
        ballot: Ballot,
        options: Optional[WrittenBallotOptions] = WrittenBallotOptions(),
    ):
        self.ballot = ballot
        self.game_results = options.game_results if options.game_results else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.ballot is not None:
            properties["ballot"] = self.ballot.properties(
            )
        if self.game_results is not None:
            properties["gameResults"] = [
                v.properties(
                )
                for v in self.game_results
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .ExperienceModelRef import ExperienceModelRef
from ..stamp_sheet.AddExperienceByUserId import AddExperienceByUserId
from ..stamp_sheet.SetExperienceByUserId import SetExperienceByUserId
from ..stamp_sheet.AddRankCapByUserId import AddRankCapByUserId
from ..stamp_sheet.SetRankCapByUserId import SetRankCapByUserId
from ..stamp_sheet.MultiplyAcquireActionsByUserId import MultiplyAcquireActionsByUserId
from ...core.model import AcquireAction
from ..stamp_sheet.SubExperienceByUserId import SubExperienceByUserId
from ..stamp_sheet.SubRankCapByUserId import SubRankCapByUserId
from ..stamp_sheet.VerifyRankByUserId import VerifyRankByUserId
from ..stamp_sheet.VerifyRankCapByUserId import VerifyRankCapByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def experience_model(
        self,
        experience_name: str,
    ) -> ExperienceModelRef:
        return ExperienceModelRef(
            self.namespace_name,
            experience_name,
        )

    def add_experience(
        self,
        experience_name: str,
        property_id: str,
        experience_value: Optional[int] = None,
        truncate_experience_when_rank_up: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AddExperienceByUserId:
        return AddExperienceByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            experience_value,
            truncate_experience_when_rank_up,
            user_id,
        )

    def set_experience(
        self,
        experience_name: str,
        property_id: str,
        experience_value: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SetExperienceByUserId:
        return SetExperienceByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            experience_value,
            user_id,
        )

    def add_rank_cap(
        self,
        experience_name: str,
        property_id: str,
        rank_cap_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> AddRankCapByUserId:
        return AddRankCapByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            rank_cap_value,
            user_id,
        )

    def set_rank_cap(
        self,
        experience_name: str,
        property_id: str,
        rank_cap_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SetRankCapByUserId:
        return SetRankCapByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            rank_cap_value,
            user_id,
        )

    def multiply_acquire_actions(
        self,
        experience_name: str,
        property_id: str,
        rate_name: str,
        acquire_actions: Optional[List[AcquireAction]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> MultiplyAcquireActionsByUserId:
        return MultiplyAcquireActionsByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            rate_name,
            acquire_actions,
            user_id,
        )

    def sub_experience(
        self,
        experience_name: str,
        property_id: str,
        experience_value: Optional[int] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SubExperienceByUserId:
        return SubExperienceByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            experience_value,
            user_id,
        )

    def sub_rank_cap(
        self,
        experience_name: str,
        property_id: str,
        rank_cap_value: int,
        user_id: Optional[str] = "#{userId}",
    ) -> SubRankCapByUserId:
        return SubRankCapByUserId(
            self.namespace_name,
            experience_name,
            property_id,
            rank_cap_value,
            user_id,
        )

    def verify_rank(
        self,
        experience_name: str,
        verify_type: str,
        property_id: str,
        rank_value: Optional[int] = None,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyRankByUserId:
        return VerifyRankByUserId(
            self.namespace_name,
            experience_name,
            verify_type,
            property_id,
            rank_value,
            multiply_value_specifying_quantity,
            user_id,
        )

    def verify_rank_cap(
        self,
        experience_name: str,
        verify_type: str,
        property_id: str,
        rank_cap_value: int,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifyRankCapByUserId:
        return VerifyRankCapByUserId(
            self.namespace_name,
            experience_name,
            verify_type,
            property_id,
            rank_cap_value,
            multiply_value_specifying_quantity,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "experience",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .AddExperienceByUserId import AddExperienceByUserId
from .SetExperienceByUserId import SetExperienceByUserId
from .AddRankCapByUserId import AddRankCapByUserId
from .SetRankCapByUserId import SetRankCapByUserId
from .MultiplyAcquireActionsByUserId import MultiplyAcquireActionsByUserId
from .SubExperienceByUserId import SubExperienceByUserId
from .SubRankCapByUserId import SubRankCapByUserId
from .VerifyRankByUserId import VerifyRankByUserId
from .VerifyRankCapByUserId import VerifyRankCapByUserId


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .NamespaceRef import NamespaceRef
from .GlobalRankingModelRef import GlobalRankingModelRef
from .GlobalRankingSeasonRef import GlobalRankingSeasonRef
from .GlobalRankingBorderRef import GlobalRankingBorderRef
from .ClusterRankingModelRef import ClusterRankingModelRef
from .ClusterRankingSeasonRef import ClusterRankingSeasonRef
from .ClusterRankingBorderRef import ClusterRankingBorderRef
from .SubscribeRankingModelRef import SubscribeRankingModelRef


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import ScriptSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef
from .enum.NamespacePriority import NamespacePriority
from .enum.NamespaceCurrency import NamespaceCurrency

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    priority: NamespacePriority
    share_free: bool
    currency: NamespaceCurrency
    description: Optional[str] = None
    apple_key: Optional[str] = None
    google_key: Optional[str] = None
    enable_fake_receipt: Optional[bool] = None
    create_wallet_script: Optional[ScriptSetting] = None
    deposit_script: Optional[ScriptSetting] = None
    withdraw_script: Optional[ScriptSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        priority: NamespacePriority,
        share_free: bool,
        currency: NamespaceCurrency,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "Money_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.priority = priority
        self.share_free = share_free
        self.currency = currency
        self.description = options.description if options.description else None
        self.apple_key = options.apple_key if options.apple_key else None
        self.google_key = options.google_key if options.google_key else None
        self.enable_fake_receipt = options.enable_fake_receipt if options.enable_fake_receipt else None
        self.create_wallet_script = options.create_wallet_script if options.create_wallet_script else None
        self.deposit_script = options.deposit_script if options.deposit_script else None
        self.withdraw_script = options.withdraw_script if options.withdraw_script else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Money::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        if self.priority is not None:
            properties["Priority"] = self.priority
        if self.share_free is not None:
            properties["ShareFree"] = self.share_free
        if self.currency is not None:
            properties["Currency"] = self.currency
        if self.apple_key is not None:
            properties["AppleKey"] = self.apple_key
        if self.google_key is not None:
            properties["GoogleKey"] = self.google_key
        if self.enable_fake_receipt is not None:
            properties["EnableFakeReceipt"] = self.enable_fake_receipt
        if self.create_wallet_script is not None:
            properties["CreateWalletScript"] = self.create_wallet_script.properties(
            )
        if self.deposit_script is not None:
            properties["DepositScript"] = self.deposit_script.properties(
            )
        if self.withdraw_script is not None:
            properties["WithdrawScript"] = self.withdraw_script.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite
from __future__ import annotations
from typing import *
from .options.AttachSecurityPolicyOptions import AttachSecurityPolicyOptions
from ...core import CdkResource, Stack


class AttachSecurityPolicy(CdkResource):

    stack: Stack
    user_name: str
    security_policy_id: str

    def __init__(
        self,
        stack: Stack,
        user_name: str,
        security_policy_id: str,
        resource_name: str = None,
    ):
        self.stack = stack
        self.user_name = user_name
        self.security_policy_id = security_policy_id
        if resource_name is None:
            resource_name = self.default_resource_name()

        super().__init__(resource_name)
        stack.add_resource(self)

    def alternate_keys(self):
        return self.user_name

    def resource_type(self) -> str:
        return "GS2::Identifier::AttachSecurityPolicy"

    def properties(self) -> Dict[str, Any]:
        return {
            "userName": self.user_name,
            "securityPolicyId": self.security_policy_id,
        }



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from ...core.model import NotificationSetting
from ...core.model import LogSetting

from ..ref.NamespaceRef import NamespaceRef

from .options.NamespaceOptions import NamespaceOptions


class Namespace(CdkResource):
    stack: Stack
    name: str
    description: Optional[str] = None
    push_notification: Optional[NotificationSetting] = None
    run_notification: Optional[NotificationSetting] = None
    log_setting: Optional[LogSetting] = None

    def __init__(
        self,
        stack: Stack,
        name: str,
        options: Optional[NamespaceOptions] = NamespaceOptions(),
    ):
        super().__init__(
            "JobQueue_Namespace_" + name
        )

        self.stack = stack
        self.name = name
        self.description = options.description if options.description else None
        self.push_notification = options.push_notification if options.push_notification else None
        self.run_notification = options.run_notification if options.run_notification else None
        self.log_setting = options.log_setting if options.log_setting else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::JobQueue::Namespace"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["Name"] = self.name
        if self.description is not None:
            properties["Description"] = self.description
        properties["EnableAutoRun"] = True
        if self.push_notification is not None:
            properties["PushNotification"] = self.push_notification.properties(
            )
        if self.run_notification is not None:
            properties["RunNotification"] = self.run_notification.properties(
            )
        if self.log_setting is not None:
            properties["LogSetting"] = self.log_setting.properties(
            )

        return properties

    def ref(
        self,
    ) -> NamespaceRef:
        return NamespaceRef(
            self.name,
        )

    def get_attr_namespace_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.NamespaceId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from ...core.model import AcquireAction
from .options.PrizeOptions import PrizeOptions
from .options.PrizeTypeIsActionOptions import PrizeTypeIsActionOptions
from .options.PrizeTypeIsPrizeTableOptions import PrizeTypeIsPrizeTableOptions
from .enum.PrizeType import PrizeType


class Prize:
    prize_id: str
    type: PrizeType
    weight: int
    acquire_actions: Optional[List[AcquireAction]] = None
    drawn_limit: Optional[int] = None
    limit_fail_over_prize_id: Optional[str] = None
    prize_table_name: Optional[str] = None

    def __init__(
        self,
        prize_id: str,
        type: PrizeType,
        weight: int,
        options: Optional[PrizeOptions] = PrizeOptions(),
    ):
        self.prize_id = prize_id
        self.type = type
        self.weight = weight
        self.acquire_actions = options.acquire_actions if options.acquire_actions else None
        self.drawn_limit = options.drawn_limit if options.drawn_limit else None
        self.limit_fail_over_prize_id = options.limit_fail_over_prize_id if options.limit_fail_over_prize_id else None
        self.prize_table_name = options.prize_table_name if options.prize_table_name else None

    @staticmethod
    def type_is_action(
        prize_id: str,
        weight: int,
        acquire_actions: List[AcquireAction],
        options: Optional[PrizeTypeIsActionOptions] = PrizeTypeIsActionOptions(),
    ) -> Prize:
        return Prize(
            prize_id,
            PrizeType.ACTION,
            weight,
            PrizeOptions(
                acquire_actions,
                options.drawn_limit,
            ),
        )

    @staticmethod
    def type_is_prize_table(
        prize_id: str,
        weight: int,
        prize_table_name: str,
        options: Optional[PrizeTypeIsPrizeTableOptions] = PrizeTypeIsPrizeTableOptions(),
    ) -> Prize:
        return Prize(
            prize_id,
            PrizeType.PRIZE_TABLE,
            weight,
            PrizeOptions(
                prize_table_name,
                options.drawn_limit,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.prize_id is not None:
            properties["prizeId"] = self.prize_id
        if self.type is not None:
            properties["type"] = self.type.value
        if self.acquire_actions is not None:
            properties["acquireActions"] = [
                v.properties(
                )
                for v in self.acquire_actions
            ]
        if self.drawn_limit is not None:
            properties["drawnLimit"] = self.drawn_limit
        if self.limit_fail_over_prize_id is not None:
            properties["limitFailOverPrizeId"] = self.limit_fail_over_prize_id
        if self.prize_table_name is not None:
            properties["prizeTableName"] = self.prize_table_name
        if self.weight is not None:
            properties["weight"] = self.weight

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .FixedTiming import FixedTiming
from .Scope import Scope
from .GlobalRankingSetting import GlobalRankingSetting
from .options.CategoryModelOptions import CategoryModelOptions
from .options.CategoryModelScopeIsGlobalOptions import CategoryModelScopeIsGlobalOptions
from .options.CategoryModelScopeIsScopedOptions import CategoryModelScopeIsScopedOptions
from .enum.CategoryModelOrderDirection import CategoryModelOrderDirection
from .enum.CategoryModelScope import CategoryModelScope


class CategoryModel:
    name: str
    sum: bool
    order_direction: CategoryModelOrderDirection
    scope: CategoryModelScope
    metadata: Optional[str] = None
    minimum_value: Optional[int] = None
    maximum_value: Optional[int] = None
    global_ranking_setting: Optional[GlobalRankingSetting] = None
    entry_period_event_id: Optional[str] = None
    access_period_event_id: Optional[str] = None
    unique_by_user_id: Optional[bool] = None
    calculate_fixed_timing_hour: Optional[int] = None
    calculate_fixed_timing_minute: Optional[int] = None
    calculate_interval_minutes: Optional[int] = None
    additional_scopes: Optional[List[Scope]] = None
    ignore_user_ids: Optional[List[str]] = None
    generation: Optional[str] = None

    def __init__(
        self,
        name: str,
        sum: bool,
        order_direction: CategoryModelOrderDirection,
        scope: CategoryModelScope,
        options: Optional[CategoryModelOptions] = CategoryModelOptions(),
    ):
        self.name = name
        self.sum = sum
        self.order_direction = order_direction
        self.scope = scope
        self.metadata = options.metadata if options.metadata else None
        self.minimum_value = options.minimum_value if options.minimum_value else None
        self.maximum_value = options.maximum_value if options.maximum_value else None
        self.global_ranking_setting = options.global_ranking_setting if options.global_ranking_setting else None
        self.entry_period_event_id = options.entry_period_event_id if options.entry_period_event_id else None
        self.access_period_event_id = options.access_period_event_id if options.access_period_event_id else None
        self.unique_by_user_id = options.unique_by_user_id if options.unique_by_user_id else None
        self.calculate_fixed_timing_hour = options.calculate_fixed_timing_hour if options.calculate_fixed_timing_hour else None
        self.calculate_fixed_timing_minute = options.calculate_fixed_timing_minute if options.calculate_fixed_timing_minute else None
        self.calculate_interval_minutes = options.calculate_interval_minutes if options.calculate_interval_minutes else None
        self.additional_scopes = options.additional_scopes if options.additional_scopes else None
        self.ignore_user_ids = options.ignore_user_ids if options.ignore_user_ids else None
        self.generation = options.generation if options.generation else None

    @staticmethod
    def scope_is_global(
        name: str,
        sum: bool,
        order_direction: CategoryModelOrderDirection,
        global_ranking_setting: GlobalRankingSetting,
        unique_by_user_id: bool,
        calculate_interval_minutes: int,
        options: Optional[CategoryModelScopeIsGlobalOptions] = CategoryModelScopeIsGlobalOptions(),
    ) -> CategoryModel:
        return CategoryModel(
            name,
            sum,
            order_direction,
            CategoryModelScope.GLOBAL,
            CategoryModelOptions(
                global_ranking_setting,
                unique_by_user_id,
                calculate_interval_minutes,
                options.metadata,
                options.minimum_value,
                options.maximum_value,
                options.entry_period_event_id,
                options.access_period_event_id,
                options.calculate_fixed_timing_hour,
                options.calculate_fixed_timing_minute,
                options.additional_scopes,
                options.ignore_user_ids,
                options.generation,
            ),
        )

    @staticmethod
    def scope_is_scoped(
        name: str,
        sum: bool,
        order_direction: CategoryModelOrderDirection,
        options: Optional[CategoryModelScopeIsScopedOptions] = CategoryModelScopeIsScopedOptions(),
    ) -> CategoryModel:
        return CategoryModel(
            name,
            sum,
            order_direction,
            CategoryModelScope.SCOPED,
            CategoryModelOptions(
                options.metadata,
                options.minimum_value,
                options.maximum_value,
                options.entry_period_event_id,
                options.access_period_event_id,
                options.calculate_fixed_timing_hour,
                options.calculate_fixed_timing_minute,
                options.additional_scopes,
                options.ignore_user_ids,
                options.generation,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.minimum_value is not None:
            properties["minimumValue"] = self.minimum_value
        if self.maximum_value is not None:
            properties["maximumValue"] = self.maximum_value
        if self.sum is not None:
            properties["sum"] = self.sum
        if self.order_direction is not None:
            properties["orderDirection"] = self.order_direction.value
        if self.scope is not None:
            properties["scope"] = self.scope.value
        if self.global_ranking_setting is not None:
            properties["globalRankingSetting"] = self.global_ranking_setting.properties(
            )
        if self.entry_period_event_id is not None:
            properties["entryPeriodEventId"] = self.entry_period_event_id
        if self.access_period_event_id is not None:
            properties["accessPeriodEventId"] = self.access_period_event_id
        if self.unique_by_user_id is not None:
            properties["uniqueByUserId"] = self.unique_by_user_id
        if self.calculate_fixed_timing_hour is not None:
            properties["calculateFixedTimingHour"] = self.calculate_fixed_timing_hour
        if self.calculate_fixed_timing_minute is not None:
            properties["calculateFixedTimingMinute"] = self.calculate_fixed_timing_minute
        if self.calculate_interval_minutes is not None:
            properties["calculateIntervalMinutes"] = self.calculate_interval_minutes
        if self.additional_scopes is not None:
            properties["additionalScopes"] = [
                v.properties(
                )
                for v in self.additional_scopes
            ]
        if self.ignore_user_ids is not None:
            properties["ignoreUserIds"] = self.ignore_user_ids
        if self.generation is not None:
            properties["generation"] = self.generation

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.RepeatScheduleOptions import RepeatScheduleOptions


class RepeatSchedule:
    repeat_count: int
    current_repeat_start_at: Optional[int] = None
    current_repeat_end_at: Optional[int] = None
    last_repeat_end_at: Optional[int] = None
    next_repeat_start_at: Optional[int] = None

    def __init__(
        self,
        repeat_count: int,
        options: Optional[RepeatScheduleOptions] = RepeatScheduleOptions(),
    ):
        self.repeat_count = repeat_count
        self.current_repeat_start_at = options.current_repeat_start_at if options.current_repeat_start_at else None
        self.current_repeat_end_at = options.current_repeat_end_at if options.current_repeat_end_at else None
        self.last_repeat_end_at = options.last_repeat_end_at if options.last_repeat_end_at else None
        self.next_repeat_start_at = options.next_repeat_start_at if options.next_repeat_start_at else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.repeat_count is not None:
            properties["repeatCount"] = self.repeat_count
        if self.current_repeat_start_at is not None:
            properties["currentRepeatStartAt"] = self.current_repeat_start_at
        if self.current_repeat_end_at is not None:
            properties["currentRepeatEndAt"] = self.current_repeat_end_at
        if self.last_repeat_end_at is not None:
            properties["lastRepeatEndAt"] = self.last_repeat_end_at
        if self.next_repeat_start_at is not None:
            properties["nextRepeatStartAt"] = self.next_repeat_start_at

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .CategoryModel import CategoryModel
from .options.CategoryModelOptions import CategoryModelOptions
from .enum.CategoryModelOrderDirection import CategoryModelOrderDirection
from .enum.CategoryModelScope import CategoryModelScope
from .options.CategoryModelScopeIsGlobalOptions import CategoryModelScopeIsGlobalOptions
from .options.CategoryModelScopeIsScopedOptions import CategoryModelScopeIsScopedOptions
from .Scope import Scope
from .options.ScopeOptions import ScopeOptions
from .GlobalRankingSetting import GlobalRankingSetting
from .options.GlobalRankingSettingOptions import GlobalRankingSettingOptions
from .FixedTiming import FixedTiming
from .options.FixedTimingOptions import FixedTimingOptions
from .CalculatedAt import CalculatedAt
from .options.CalculatedAtOptions import CalculatedAtOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
#
# deny overwrite
from __future__ import annotations
from typing import *
from .options.LotteryModelOptions import LotteryModelOptions
from .options.LotteryModelMethodIsPrizeTableOptions import LotteryModelMethodIsPrizeTableOptions
from .options.LotteryModelMethodIsScriptOptions import LotteryModelMethodIsScriptOptions
from .enum.LotteryModelMode import LotteryModelMode
from .enum.LotteryModelMethod import LotteryModelMethod


class LotteryModel:
    name: str
    mode: LotteryModelMode
    method: LotteryModelMethod
    metadata: Optional[str] = None
    prize_table_name: Optional[str] = None
    choice_prize_table_script_id: Optional[str] = None

    def __init__(
        self,
        name: str,
        mode: LotteryModelMode,
        method: LotteryModelMethod,
        options: Optional[LotteryModelOptions] = LotteryModelOptions(),
    ):
        self.name = name
        self.mode = mode
        self.method = method
        self.metadata = options.metadata if options.metadata else None
        self.prize_table_name = options.prize_table_name if options.prize_table_name else None
        self.choice_prize_table_script_id = options.choice_prize_table_script_id if options.choice_prize_table_script_id else None

    @staticmethod
    def method_is_prize_table(
        name: str,
        mode: LotteryModelMode,
        options: Optional[LotteryModelMethodIsPrizeTableOptions] = LotteryModelMethodIsPrizeTableOptions(),
    ) -> LotteryModel:
        return LotteryModel(
            name,
            mode,
            LotteryModelMethod.PRIZE_TABLE,
            LotteryModelOptions(
                options.metadata,
                options.prize_table_name,
            ),
        )

    @staticmethod
    def method_is_script(
        name: str,
        mode: LotteryModelMode,
        options: Optional[LotteryModelMethodIsScriptOptions] = LotteryModelMethodIsScriptOptions(),
    ) -> LotteryModel:
        return LotteryModel(
            name,
            mode,
            LotteryModelMethod.SCRIPT,
            LotteryModelOptions(
                options.metadata,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.mode is not None:
            properties["mode"] = self.mode.value
        if self.method is not None:
            properties["method"] = self.method.value
        if self.prize_table_name is not None:
            properties["prizeTableName"] = self.prize_table_name
        if self.choice_prize_table_script_id is not None:
            properties["choicePrizeTableScriptId"] = self.choice_prize_table_script_id

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.model import CdkResource, Stack
from ...core.func import GetAttr
from .RoleModel import RoleModel

from ..ref.GuildRef import GuildRef
from .enum.GuildJoinPolicy import GuildJoinPolicy

from .options.GuildOptions import GuildOptions


class Guild(CdkResource):
    stack: Stack
    namespace_name: str
    user_id: str
    guild_model_name: str
    display_name: str
    join_policy: GuildJoinPolicy
    attribute1: Optional[int] = None
    attribute2: Optional[int] = None
    attribute3: Optional[int] = None
    attribute4: Optional[int] = None
    attribute5: Optional[int] = None
    custom_roles: Optional[List[RoleModel]] = None
    guild_member_default_role: Optional[str] = None
    time_offset_token: Optional[str] = None

    def __init__(
        self,
        stack: Stack,
        namespace_name: str,
        user_id: str,
        guild_model_name: str,
        display_name: str,
        join_policy: GuildJoinPolicy,
        options: Optional[GuildOptions] = GuildOptions(),
    ):
        super().__init__(
            "Guild_Guild_" + guild_model_name + ":" + name
        )

        self.stack = stack
        self.namespace_name = namespace_name
        self.user_id = user_id
        self.guild_model_name = guild_model_name
        self.display_name = display_name
        self.join_policy = join_policy
        self.attribute1 = options.attribute1 if options.attribute1 else None
        self.attribute2 = options.attribute2 if options.attribute2 else None
        self.attribute3 = options.attribute3 if options.attribute3 else None
        self.attribute4 = options.attribute4 if options.attribute4 else None
        self.attribute5 = options.attribute5 if options.attribute5 else None
        self.custom_roles = options.custom_roles if options.custom_roles else None
        self.guild_member_default_role = options.guild_member_default_role if options.guild_member_default_role else None
        self.time_offset_token = options.time_offset_token if options.time_offset_token else None
        stack.add_resource(
            self,
        )


    def alternate_keys(
        self,
    ):
        return "guild_model_name + ":" + name"

    def resource_type(
        self,
    ) -> str:
        return "GS2::Guild::Guild"

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.namespace_name is not None:
            properties["NamespaceName"] = self.namespace_name
        if self.user_id is not None:
            properties["UserId"] = self.user_id
        if self.guild_model_name is not None:
            properties["GuildModelName"] = self.guild_model_name
        if self.display_name is not None:
            properties["DisplayName"] = self.display_name
        if self.attribute1 is not None:
            properties["Attribute1"] = self.attribute1
        if self.attribute2 is not None:
            properties["Attribute2"] = self.attribute2
        if self.attribute3 is not None:
            properties["Attribute3"] = self.attribute3
        if self.attribute4 is not None:
            properties["Attribute4"] = self.attribute4
        if self.attribute5 is not None:
            properties["Attribute5"] = self.attribute5
        if self.join_policy is not None:
            properties["JoinPolicy"] = self.join_policy
        if self.custom_roles is not None:
            properties["CustomRoles"] = [
                v.properties(
                )
                for v in self.custom_roles
            ]
        if self.guild_member_default_role is not None:
            properties["GuildMemberDefaultRole"] = self.guild_member_default_role
        if self.time_offset_token is not None:
            properties["TimeOffsetToken"] = self.time_offset_token

        return properties

    def ref(
        self,
        name: str,
    ) -> GuildRef:
        return GuildRef(
            self.namespace_name,
            self.guild_model_name,
            name,
        )

    def get_attr_guild_id(
        self,
    ) -> GetAttr:
        return GetAttr(
            self,
            "Item.GuildId",
            None,
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from ..stamp_sheet.AcquireActionsToFormProperties import AcquireActionsToFormProperties
from ...core.model import AcquireAction
from ...core.model import Config
from ..stamp_sheet.SetFormByUserId import SetFormByUserId


class FormModelRef:
    namespace_name: str
    mold_model_name: str

    def __init__(
        self,
        namespace_name: str,
        mold_model_name: str,
    ):
        self.namespace_name = namespace_name
        self.mold_model_name = mold_model_name

    def acquire_actions_to_form_properties(
        self,
        index: int,
        acquire_action: AcquireAction,
        config: Optional[List[Config]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireActionsToFormProperties:
        return AcquireActionsToFormProperties(
            self.namespace_name,
            self.mold_model_name,
            index,
            acquire_action,
            config,
            user_id,
        )

    def set_form(
        self,
        index: int,
        slots: List[Slot],
        user_id: Optional[str] = "#{userId}",
    ) -> SetFormByUserId:
        return SetFormByUserId(
            self.namespace_name,
            self.mold_model_name,
            index,
            slots,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "formation",
                self.namespace_name,
                "model",
                "mold",
                self.mold_model_name,
                "form",
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .BonusRate import BonusRate
from .options.RateModelOptions import RateModelOptions


class RateModel:
    name: str
    target_inventory_model_id: str
    acquire_experience_suffix: str
    material_inventory_model_id: str
    experience_model_id: str
    description: Optional[str] = None
    metadata: Optional[str] = None
    acquire_experience_hierarchy: Optional[List[str]] = None
    bonus_rates: Optional[List[BonusRate]] = None

    def __init__(
        self,
        name: str,
        target_inventory_model_id: str,
        acquire_experience_suffix: str,
        material_inventory_model_id: str,
        experience_model_id: str,
        options: Optional[RateModelOptions] = RateModelOptions(),
    ):
        self.name = name
        self.target_inventory_model_id = target_inventory_model_id
        self.acquire_experience_suffix = acquire_experience_suffix
        self.material_inventory_model_id = material_inventory_model_id
        self.experience_model_id = experience_model_id
        self.description = options.description if options.description else None
        self.metadata = options.metadata if options.metadata else None
        self.acquire_experience_hierarchy = options.acquire_experience_hierarchy if options.acquire_experience_hierarchy else None
        self.bonus_rates = options.bonus_rates if options.bonus_rates else None

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.name is not None:
            properties["name"] = self.name
        if self.description is not None:
            properties["description"] = self.description
        if self.metadata is not None:
            properties["metadata"] = self.metadata
        if self.target_inventory_model_id is not None:
            properties["targetInventoryModelId"] = self.target_inventory_model_id
        if self.acquire_experience_suffix is not None:
            properties["acquireExperienceSuffix"] = self.acquire_experience_suffix
        if self.material_inventory_model_id is not None:
            properties["materialInventoryModelId"] = self.material_inventory_model_id
        if self.acquire_experience_hierarchy is not None:
            properties["acquireExperienceHierarchy"] = self.acquire_experience_hierarchy
        if self.experience_model_id is not None:
            properties["experienceModelId"] = self.experience_model_id
        if self.bonus_rates is not None:
            properties["bonusRates"] = [
                v.properties(
                )
                for v in self.bonus_rates
            ]

        return properties



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from .Namespace import Namespace
from .options.NamespaceOptions import NamespaceOptions
from .BuffTargetModel import BuffTargetModel
from .options.BuffTargetModelOptions import BuffTargetModelOptions
from .BuffTargetAction import BuffTargetAction
from .options.BuffTargetActionOptions import BuffTargetActionOptions
from .BuffTargetGrn import BuffTargetGrn
from .options.BuffTargetGrnOptions import BuffTargetGrnOptions
from .BuffEntryModel import BuffEntryModel
from .options.BuffEntryModelOptions import BuffEntryModelOptions
from .enum.BuffEntryModelTargetType import BuffEntryModelTargetType
from .enum.BuffEntryModelExpression import BuffEntryModelExpression
from .options.BuffEntryModelTargetTypeIsModelOptions import BuffEntryModelTargetTypeIsModelOptions
from .options.BuffEntryModelTargetTypeIsActionOptions import BuffEntryModelTargetTypeIsActionOptions
from .OverrideBuffRate import OverrideBuffRate
from .options.OverrideBuffRateOptions import OverrideBuffRateOptions
from .CurrentMasterData import CurrentMasterData


# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .RateModelRef import RateModelRef
from .IncrementalRateModelRef import IncrementalRateModelRef
from ..stamp_sheet.ExchangeByUserId import ExchangeByUserId
from ...core.model import Config
from ..stamp_sheet.IncrementalExchangeByUserId import IncrementalExchangeByUserId
from ..stamp_sheet.UnlockIncrementalExchangeByUserId import UnlockIncrementalExchangeByUserId
from ..stamp_sheet.CreateAwaitByUserId import CreateAwaitByUserId
from ..stamp_sheet.SkipByUserId import SkipByUserId
from ..stamp_sheet.DeleteAwaitByUserId import DeleteAwaitByUserId


class NamespaceRef:
    namespace_name: str

    def __init__(
        self,
        namespace_name: str,
    ):
        self.namespace_name = namespace_name

    def rate_model(
        self,
        rate_name: str,
    ) -> RateModelRef:
        return RateModelRef(
            self.namespace_name,
            rate_name,
        )

    def incremental_rate_model(
        self,
        rate_name: str,
    ) -> IncrementalRateModelRef:
        return IncrementalRateModelRef(
            self.namespace_name,
            rate_name,
        )

    def exchange(
        self,
        rate_name: str,
        count: int,
        config: Optional[List[Config]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> ExchangeByUserId:
        return ExchangeByUserId(
            self.namespace_name,
            rate_name,
            count,
            config,
            user_id,
        )

    def incremental_exchange(
        self,
        rate_name: str,
        count: int,
        config: Optional[List[Config]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> IncrementalExchangeByUserId:
        return IncrementalExchangeByUserId(
            self.namespace_name,
            rate_name,
            count,
            config,
            user_id,
        )

    def unlock_incremental_exchange(
        self,
        rate_name: str,
        lock_transaction_id: str,
        user_id: Optional[str] = "#{userId}",
    ) -> UnlockIncrementalExchangeByUserId:
        return UnlockIncrementalExchangeByUserId(
            self.namespace_name,
            rate_name,
            lock_transaction_id,
            user_id,
        )

    def create_await(
        self,
        rate_name: str,
        count: Optional[int] = None,
        config: Optional[List[Config]] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> CreateAwaitByUserId:
        return CreateAwaitByUserId(
            self.namespace_name,
            rate_name,
            count,
            config,
            user_id,
        )

    def skip(
        self,
        await_name: Optional[str] = None,
        skip_type: Optional[str] = None,
        minutes: Optional[int] = None,
        rate: Optional[float] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> SkipByUserId:
        return SkipByUserId(
            self.namespace_name,
            await_name,
            skip_type,
            minutes,
            rate,
            user_id,
        )

    def delete_await(
        self,
        await_name: Optional[str] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> DeleteAwaitByUserId:
        return DeleteAwaitByUserId(
            self.namespace_name,
            await_name,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "exchange",
                self.namespace_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *

from ...core.func import GetAttr, Join
from .SimpleItemModelRef import SimpleItemModelRef
from ..stamp_sheet.AcquireSimpleItemsByUserId import AcquireSimpleItemsByUserId
from ..model.AcquireCount import AcquireCount
from ..stamp_sheet.SetSimpleItemsByUserId import SetSimpleItemsByUserId
from ..stamp_sheet.ConsumeSimpleItemsByUserId import ConsumeSimpleItemsByUserId
from ..stamp_sheet.VerifySimpleItemByUserId import VerifySimpleItemByUserId


class SimpleInventoryModelRef:
    namespace_name: str
    inventory_name: str

    def __init__(
        self,
        namespace_name: str,
        inventory_name: str,
    ):
        self.namespace_name = namespace_name
        self.inventory_name = inventory_name

    def simple_item_model(
        self,
        item_name: str,
    ) -> SimpleItemModelRef:
        return SimpleItemModelRef(
            self.namespace_name,
            self.inventory_name,
            item_name,
        )

    def acquire_simple_items(
        self,
        acquire_counts: List[AcquireCount],
        user_id: Optional[str] = "#{userId}",
    ) -> AcquireSimpleItemsByUserId:
        return AcquireSimpleItemsByUserId(
            self.namespace_name,
            self.inventory_name,
            acquire_counts,
            user_id,
        )

    def set_simple_items(
        self,
        counts: List[HeldCount],
        user_id: Optional[str] = "#{userId}",
    ) -> SetSimpleItemsByUserId:
        return SetSimpleItemsByUserId(
            self.namespace_name,
            self.inventory_name,
            counts,
            user_id,
        )

    def consume_simple_items(
        self,
        consume_counts: List[ConsumeCount],
        user_id: Optional[str] = "#{userId}",
    ) -> ConsumeSimpleItemsByUserId:
        return ConsumeSimpleItemsByUserId(
            self.namespace_name,
            self.inventory_name,
            consume_counts,
            user_id,
        )

    def verify_simple_item(
        self,
        item_name: str,
        verify_type: str,
        count: int,
        multiply_value_specifying_quantity: Optional[bool] = None,
        user_id: Optional[str] = "#{userId}",
    ) -> VerifySimpleItemByUserId:
        return VerifySimpleItemByUserId(
            self.namespace_name,
            self.inventory_name,
            item_name,
            verify_type,
            count,
            multiply_value_specifying_quantity,
            user_id,
        )

    def grn(
        self,
    ) -> str:
        return Join(
            ":",
            [
                "grn",
                "gs2",
                GetAttr.region(
                ).str(
                ),
                GetAttr.owner_id(
                ).str(
                ),
                "inventory",
                self.namespace_name,
                "simple",
                "model",
                self.inventory_name,
            ],
        ).str(
        )



# Copyright 2016- Game Server Services, Inc. or its affiliates. All Rights
# Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License").
# You may not use this file except in compliance with the License.
# A copy of the License is located at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# or in the "license" file accompanying this file. This file is distributed
# on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
# express or implied. See the License for the specific language governing
# permissions and limitations under the License.
from __future__ import annotations
from typing import *
from .options.CounterScopeModelOptions import CounterScopeModelOptions
from .options.CounterScopeModelResetTypeIsNotResetOptions import CounterScopeModelResetTypeIsNotResetOptions
from .options.CounterScopeModelResetTypeIsDailyOptions import CounterScopeModelResetTypeIsDailyOptions
from .options.CounterScopeModelResetTypeIsWeeklyOptions import CounterScopeModelResetTypeIsWeeklyOptions
from .options.CounterScopeModelResetTypeIsMonthlyOptions import CounterScopeModelResetTypeIsMonthlyOptions
from .enum.CounterScopeModelResetType import CounterScopeModelResetType
from .enum.CounterScopeModelResetDayOfWeek import CounterScopeModelResetDayOfWeek


class CounterScopeModel:
    reset_type: CounterScopeModelResetType
    reset_day_of_month: Optional[int] = None
    reset_day_of_week: Optional[CounterScopeModelResetDayOfWeek] = None
    reset_hour: Optional[int] = None

    def __init__(
        self,
        reset_type: CounterScopeModelResetType,
        options: Optional[CounterScopeModelOptions] = CounterScopeModelOptions(),
    ):
        self.reset_type = reset_type
        self.reset_day_of_month = options.reset_day_of_month if options.reset_day_of_month else None
        self.reset_day_of_week = options.reset_day_of_week if options.reset_day_of_week else None
        self.reset_hour = options.reset_hour if options.reset_hour else None

    @staticmethod
    def reset_type_is_not_reset(
        options: Optional[CounterScopeModelResetTypeIsNotResetOptions] = CounterScopeModelResetTypeIsNotResetOptions(),
    ) -> CounterScopeModel:
        return CounterScopeModel(
            CounterScopeModelResetType.NOT_RESET,
            CounterScopeModelOptions(
            ),
        )

    @staticmethod
    def reset_type_is_daily(
        reset_hour: int,
        options: Optional[CounterScopeModelResetTypeIsDailyOptions] = CounterScopeModelResetTypeIsDailyOptions(),
    ) -> CounterScopeModel:
        return CounterScopeModel(
            CounterScopeModelResetType.DAILY,
            CounterScopeModelOptions(
                reset_hour,
            ),
        )

    @staticmethod
    def reset_type_is_weekly(
        reset_day_of_week: CounterScopeModelResetDayOfWeek,
        reset_hour: int,
        options: Optional[CounterScopeModelResetTypeIsWeeklyOptions] = CounterScopeModelResetTypeIsWeeklyOptions(),
    ) -> CounterScopeModel:
        return CounterScopeModel(
            CounterScopeModelResetType.WEEKLY,
            CounterScopeModelOptions(
                reset_day_of_week,
                reset_hour,
            ),
        )

    @staticmethod
    def reset_type_is_monthly(
        reset_day_of_month: int,
        reset_hour: int,
        options: Optional[CounterScopeModelResetTypeIsMonthlyOptions] = CounterScopeModelResetTypeIsMonthlyOptions(),
    ) -> CounterScopeModel:
        return CounterScopeModel(
            CounterScopeModelResetType.MONTHLY,
            CounterScopeModelOptions(
                reset_day_of_month,
                reset_hour,
            ),
        )

    def properties(
        self,
    ) -> Dict[str, Any]:
        properties: Dict[str, Any] = {}

        if self.reset_type is not None:
            properties["resetType"] = self.reset_type.value
        if self.reset_day_of_month is not None:
            properties["resetDayOfMonth"] = self.reset_day_of_month
        if self.reset_day_of_week is not None:
            properties["resetDayOfWeek"] = self.reset_day_of_week.value
        if self.reset_hour is not None:
            properties["resetHour"] = self.reset_hour

        return properties
